"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .squadcastsdkerror import SquadcastSDKError
from typing import TYPE_CHECKING
from importlib import import_module
import builtins
import sys

if TYPE_CHECKING:
    from .analytics_getorganalyticsop import (
        BadGatewayError,
        BadGatewayErrorData,
        BadRequest,
        BadRequestError,
        BadRequestErrorData,
        BadRequestUnion,
        ConflictError,
        ConflictErrorData,
        ForbiddenError,
        ForbiddenErrorData,
        GatewayTimeoutError,
        GatewayTimeoutErrorData,
        InternalServerError,
        InternalServerErrorData,
        NotFoundError,
        NotFoundErrorData,
        PaymentRequiredError,
        PaymentRequiredErrorData,
        ResponseBodyError1,
        ResponseBodyError1Data,
        ResponseBodyError2,
        ResponseBodyError2Data,
        ServiceUnavailableError,
        ServiceUnavailableErrorData,
        UnauthorizedError,
        UnauthorizedErrorData,
        UnprocessableEntityError,
        UnprocessableEntityErrorData,
    )
    from .common_v4_error import CommonV4Error, CommonV4ErrorData
    from .no_response_error import NoResponseError
    from .responsevalidationerror import ResponseValidationError
    from .sdkdefaulterror import SDKDefaultError

__all__ = [
    "BadGatewayError",
    "BadGatewayErrorData",
    "BadRequest",
    "BadRequestError",
    "BadRequestErrorData",
    "BadRequestUnion",
    "CommonV4Error",
    "CommonV4ErrorData",
    "ConflictError",
    "ConflictErrorData",
    "ForbiddenError",
    "ForbiddenErrorData",
    "GatewayTimeoutError",
    "GatewayTimeoutErrorData",
    "InternalServerError",
    "InternalServerErrorData",
    "NoResponseError",
    "NotFoundError",
    "NotFoundErrorData",
    "PaymentRequiredError",
    "PaymentRequiredErrorData",
    "ResponseBodyError1",
    "ResponseBodyError1Data",
    "ResponseBodyError2",
    "ResponseBodyError2Data",
    "ResponseValidationError",
    "SDKDefaultError",
    "ServiceUnavailableError",
    "ServiceUnavailableErrorData",
    "SquadcastSDKError",
    "UnauthorizedError",
    "UnauthorizedErrorData",
    "UnprocessableEntityError",
    "UnprocessableEntityErrorData",
]

_dynamic_imports: dict[str, str] = {
    "BadGatewayError": ".analytics_getorganalyticsop",
    "BadGatewayErrorData": ".analytics_getorganalyticsop",
    "BadRequest": ".analytics_getorganalyticsop",
    "BadRequestError": ".analytics_getorganalyticsop",
    "BadRequestErrorData": ".analytics_getorganalyticsop",
    "BadRequestUnion": ".analytics_getorganalyticsop",
    "ConflictError": ".analytics_getorganalyticsop",
    "ConflictErrorData": ".analytics_getorganalyticsop",
    "ForbiddenError": ".analytics_getorganalyticsop",
    "ForbiddenErrorData": ".analytics_getorganalyticsop",
    "GatewayTimeoutError": ".analytics_getorganalyticsop",
    "GatewayTimeoutErrorData": ".analytics_getorganalyticsop",
    "InternalServerError": ".analytics_getorganalyticsop",
    "InternalServerErrorData": ".analytics_getorganalyticsop",
    "NotFoundError": ".analytics_getorganalyticsop",
    "NotFoundErrorData": ".analytics_getorganalyticsop",
    "PaymentRequiredError": ".analytics_getorganalyticsop",
    "PaymentRequiredErrorData": ".analytics_getorganalyticsop",
    "ResponseBodyError1": ".analytics_getorganalyticsop",
    "ResponseBodyError1Data": ".analytics_getorganalyticsop",
    "ResponseBodyError2": ".analytics_getorganalyticsop",
    "ResponseBodyError2Data": ".analytics_getorganalyticsop",
    "ServiceUnavailableError": ".analytics_getorganalyticsop",
    "ServiceUnavailableErrorData": ".analytics_getorganalyticsop",
    "UnauthorizedError": ".analytics_getorganalyticsop",
    "UnauthorizedErrorData": ".analytics_getorganalyticsop",
    "UnprocessableEntityError": ".analytics_getorganalyticsop",
    "UnprocessableEntityErrorData": ".analytics_getorganalyticsop",
    "CommonV4Error": ".common_v4_error",
    "CommonV4ErrorData": ".common_v4_error",
    "NoResponseError": ".no_response_error",
    "ResponseValidationError": ".responsevalidationerror",
    "SDKDefaultError": ".sdkdefaulterror",
}


def dynamic_import(modname, retries=3):
    for attempt in range(retries):
        try:
            return import_module(modname, __package__)
        except KeyError:
            # Clear any half-initialized module and retry
            sys.modules.pop(modname, None)
            if attempt == retries - 1:
                break
    raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = dynamic_import(module_name)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = builtins.list(_dynamic_imports.keys())
    return builtins.sorted(lazy_attrs)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsUpdateIncidentPriorityUpdateName = Literal["sq_update_incident_priority",]


V3WorkflowsUpdateIncidentPriorityUpdatePriority = Literal[
    "P1",
    "P2",
    "P3",
    "P4",
    "P5",
    "UNSET",
]


class V3WorkflowsUpdateIncidentPriorityUpdateDataTypedDict(TypedDict):
    priority: NotRequired[V3WorkflowsUpdateIncidentPriorityUpdatePriority]


class V3WorkflowsUpdateIncidentPriorityUpdateData(BaseModel):
    priority: Optional[V3WorkflowsUpdateIncidentPriorityUpdatePriority] = None


class V3WorkflowsUpdateIncidentPriorityUpdateTypedDict(TypedDict):
    name: V3WorkflowsUpdateIncidentPriorityUpdateName
    data: NotRequired[V3WorkflowsUpdateIncidentPriorityUpdateDataTypedDict]


class V3WorkflowsUpdateIncidentPriorityUpdate(BaseModel):
    name: V3WorkflowsUpdateIncidentPriorityUpdateName

    data: Optional[V3WorkflowsUpdateIncidentPriorityUpdateData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaloncallreminderrules_globaloncallreminderruleresponse import (
    V3GlobalOncallReminderRulesGlobalOncallReminderRuleResponse,
    V3GlobalOncallReminderRulesGlobalOncallReminderRuleResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class GlobalOncallReminderRulesCreateGlobalOncallReminderRulesResponseTypedDict(
    TypedDict
):
    r"""The request has succeeded."""

    data: V3GlobalOncallReminderRulesGlobalOncallReminderRuleResponseTypedDict


class GlobalOncallReminderRulesCreateGlobalOncallReminderRulesResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3GlobalOncallReminderRulesGlobalOncallReminderRuleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_slo_marksloaffectedrequest import (
    V3SLOMarkSLOAffectedRequest,
    V3SLOMarkSLOAffectedRequestTypedDict,
)
from .v3_slo_sloviolatingincidentresponse import (
    V3SLOSLOViolatingIncidentResponse,
    V3SLOSLOViolatingIncidentResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import (
    FieldMetadata,
    PathParamMetadata,
    QueryParamMetadata,
    RequestMetadata,
)
from typing_extensions import Annotated, TypedDict


class SLOMarkSLOAffectedRequestTypedDict(TypedDict):
    slo_id: int
    owner_id_param: str
    v3_slo_mark_slo_affected_request: V3SLOMarkSLOAffectedRequestTypedDict


class SLOMarkSLOAffectedRequest(BaseModel):
    slo_id: Annotated[
        int,
        pydantic.Field(alias="sloID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    owner_id_param: Annotated[
        str,
        pydantic.Field(alias="owner_id"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]

    v3_slo_mark_slo_affected_request: Annotated[
        V3SLOMarkSLOAffectedRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class SLOMarkSLOAffectedDataTypedDict(TypedDict):
    slo_violating_incident: V3SLOSLOViolatingIncidentResponseTypedDict


class SLOMarkSLOAffectedData(BaseModel):
    slo_violating_incident: V3SLOSLOViolatingIncidentResponse


class SLOMarkSLOAffectedResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: SLOMarkSLOAffectedDataTypedDict


class SLOMarkSLOAffectedResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: SLOMarkSLOAffectedData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V4StatusPagesDeleteStatusPageByIDResponseTypedDict(TypedDict):
    id: int
    name: str
    organization_id: str
    team_id: str


class V4StatusPagesDeleteStatusPageByIDResponse(BaseModel):
    id: int

    name: str

    organization_id: Annotated[str, pydantic.Field(alias="organizationID")]

    team_id: Annotated[str, pydantic.Field(alias="teamID")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Literal
from typing_extensions import TypedDict


V3WorkflowsSqSendEmailName = Literal["sq_send_email",]


class V3WorkflowsSqSendEmailDataTypedDict(TypedDict):
    to: List[str]
    subject: str
    body: str


class V3WorkflowsSqSendEmailData(BaseModel):
    to: List[str]

    subject: str

    body: str


class V3WorkflowsSqSendEmailTypedDict(TypedDict):
    name: V3WorkflowsSqSendEmailName
    data: V3WorkflowsSqSendEmailDataTypedDict


class V3WorkflowsSqSendEmail(BaseModel):
    name: V3WorkflowsSqSendEmailName

    data: V3WorkflowsSqSendEmailData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3GlobalEventRulesRuleActionTypedDict(TypedDict):
    route_to: str
    r"""The service ID to which the alert should be routed."""


class V3GlobalEventRulesRuleAction(BaseModel):
    route_to: str
    r"""The service ID to which the alert should be routed."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_auditlogs_listauditlogsresponse import (
    V3AuditLogsListAuditLogsResponse,
    V3AuditLogsListAuditLogsResponseTypedDict,
)
from datetime import date
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Callable, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


Client = Literal[
    "web",
    "mobile",
    "terraform",
]


class AuditLogsListAuditLogsRequestTypedDict(TypedDict):
    page_size: int
    page_number: int
    start_date: date
    end_date: date
    action: NotRequired[List[str]]
    resource: NotRequired[List[str]]
    actor: NotRequired[List[str]]
    team: NotRequired[List[str]]
    client: NotRequired[List[Client]]


class AuditLogsListAuditLogsRequest(BaseModel):
    page_size: Annotated[
        int,
        pydantic.Field(alias="pageSize"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    page_number: Annotated[
        int,
        pydantic.Field(alias="pageNumber"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    start_date: Annotated[
        date,
        pydantic.Field(alias="startDate"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    end_date: Annotated[
        date,
        pydantic.Field(alias="endDate"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    action: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="action[]"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    resource: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="resource[]"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    actor: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="actor[]"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    team: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="team[]"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    client: Annotated[
        Optional[List[Client]],
        pydantic.Field(alias="client[]"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None


class AuditLogsListAuditLogsResponseTypedDict(TypedDict):
    result: V3AuditLogsListAuditLogsResponseTypedDict


class AuditLogsListAuditLogsResponse(BaseModel):
    next: Callable[[], Optional[AuditLogsListAuditLogsResponse]]

    result: V3AuditLogsListAuditLogsResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ServicesExtensionsSlackExtensionResponseTypedDict(TypedDict):
    id: int
    service_id: str
    organization_id: str
    channel_id: str
    created_at: str
    name: str


class V3ServicesExtensionsSlackExtensionResponse(BaseModel):
    id: int

    service_id: str

    organization_id: str

    channel_id: str

    created_at: str

    name: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3WorkflowsSlackMessageUserName = Literal["slack_message_user",]


class V3WorkflowsSlackMessageUserDataTypedDict(TypedDict):
    member_id: str
    message: str


class V3WorkflowsSlackMessageUserData(BaseModel):
    member_id: str

    message: str


class V3WorkflowsSlackMessageUserTypedDict(TypedDict):
    name: V3WorkflowsSlackMessageUserName
    data: V3WorkflowsSlackMessageUserDataTypedDict


class V3WorkflowsSlackMessageUser(BaseModel):
    name: V3WorkflowsSlackMessageUserName

    data: V3WorkflowsSlackMessageUserData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_auditlogs_listauditlogsexporthistoryresponse import (
    V3AuditLogsListAuditLogsExportHistoryResponse,
    V3AuditLogsListAuditLogsExportHistoryResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Callable, Optional
from typing_extensions import Annotated, TypedDict


class AuditLogsListAuditLogsExportHistoryRequestTypedDict(TypedDict):
    page_size: int
    page_number: int


class AuditLogsListAuditLogsExportHistoryRequest(BaseModel):
    page_size: Annotated[
        int,
        pydantic.Field(alias="pageSize"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    page_number: Annotated[
        int,
        pydantic.Field(alias="pageNumber"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]


class AuditLogsListAuditLogsExportHistoryResponseTypedDict(TypedDict):
    result: V3AuditLogsListAuditLogsExportHistoryResponseTypedDict


class AuditLogsListAuditLogsExportHistoryResponse(BaseModel):
    next: Callable[[], Optional[AuditLogsListAuditLogsExportHistoryResponse]]

    result: V3AuditLogsListAuditLogsExportHistoryResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_getworkflowactionbyidresponse import (
    V3WorkflowsGetWorkflowActionByIDResponse,
    V3WorkflowsGetWorkflowActionByIDResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class WorkflowsGetWorkflowActionByIDRequestTypedDict(TypedDict):
    workflow_id: str
    action_id: str


class WorkflowsGetWorkflowActionByIDRequest(BaseModel):
    workflow_id: Annotated[
        str,
        pydantic.Field(alias="workflowID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    action_id: Annotated[
        str,
        pydantic.Field(alias="actionID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class WorkflowsGetWorkflowActionByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3WorkflowsGetWorkflowActionByIDResponseTypedDict


class WorkflowsGetWorkflowActionByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3WorkflowsGetWorkflowActionByIDResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3IncidentsIngestionStatusRequestTypedDict(TypedDict):
    r"""Request body for getting ingestion status."""

    request_ids: List[str]


class V3IncidentsIngestionStatusRequest(BaseModel):
    r"""Request body for getting ingestion status."""

    request_ids: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesMaintenancesMaintenanceComponentTypedDict(TypedDict):
    id: NotRequired[int]
    name: NotRequired[str]
    group_name: NotRequired[str]


class V4StatusPagesMaintenancesMaintenanceComponent(BaseModel):
    id: Optional[int] = None

    name: Optional[str] = None

    group_name: Annotated[Optional[str], pydantic.Field(alias="groupName")] = None


class V4StatusPagesMaintenancesMaintenanceTypedDict(TypedDict):
    id: NotRequired[int]
    title: NotRequired[str]
    page_id: NotRequired[int]
    note: NotRequired[str]
    components: NotRequired[
        List[V4StatusPagesMaintenancesMaintenanceComponentTypedDict]
    ]
    start_time: NotRequired[datetime]
    end_time: NotRequired[datetime]
    maintenance_state: NotRequired[str]


class V4StatusPagesMaintenancesMaintenance(BaseModel):
    id: Optional[int] = None

    title: Optional[str] = None

    page_id: Annotated[Optional[int], pydantic.Field(alias="pageID")] = None

    note: Optional[str] = None

    components: Optional[List[V4StatusPagesMaintenancesMaintenanceComponent]] = None

    start_time: Annotated[Optional[datetime], pydantic.Field(alias="startTime")] = None

    end_time: Annotated[Optional[datetime], pydantic.Field(alias="endTime")] = None

    maintenance_state: Annotated[
        Optional[str], pydantic.Field(alias="maintenanceState")
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from pydantic import model_serializer
from squadcast_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from typing import List
from typing_extensions import TypedDict


class V3SLOSLOViolatingIncidentResponseTypedDict(TypedDict):
    id: int
    owner_type: str
    owner_id: str
    org_id: str
    slo_id: int
    incident_id: str
    slis: List[str]
    error_budget_spent: float
    is_false_positive: bool
    start_time: datetime
    end_time: datetime
    created_at: datetime
    updated_at: datetime
    deleted_at: Nullable[datetime]


class V3SLOSLOViolatingIncidentResponse(BaseModel):
    id: int

    owner_type: str

    owner_id: str

    org_id: str

    slo_id: int

    incident_id: str

    slis: List[str]

    error_budget_spent: float

    is_false_positive: bool

    start_time: datetime

    end_time: datetime

    created_at: datetime

    updated_at: datetime

    deleted_at: Nullable[datetime]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["deleted_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsTagsMergeIncidentEntityReferenceTypedDict(TypedDict):
    r"""Reference to a merged incident entity."""

    id: str


class V3IncidentsTagsMergeIncidentEntityReference(BaseModel):
    r"""Reference to a merged incident entity."""

    id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsCommunicationCardsCreateCommunicationCardRequestTypedDict(TypedDict):
    r"""Request body for creating a communication card."""

    type: str
    url: str
    title: str


class V3IncidentsCommunicationCardsCreateCommunicationCardRequest(BaseModel):
    r"""Request body for creating a communication card."""

    type: str

    url: str

    title: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V4SquadsUpdateSquadMemberRequestTypedDict(TypedDict):
    role: str


class V4SquadsUpdateSquadMemberRequest(BaseModel):
    role: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_entityowner import (
    V3GlobalEventRulesEntityOwner,
    V3GlobalEventRulesEntityOwnerTypedDict,
)
from .v3_globaleventrules_ruleset import (
    V3GlobalEventRulesRuleset,
    V3GlobalEventRulesRulesetTypedDict,
)
from datetime import datetime
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3GlobalEventRulesCreatedGlobalEventRuleResponseTypedDict(TypedDict):
    id: int
    owner_id: str
    name: str
    description: str
    routing_key: str
    created_at: datetime
    created_by: str
    updated_at: datetime
    updated_by: str
    entity_owner: NotRequired[V3GlobalEventRulesEntityOwnerTypedDict]
    rulesets: NotRequired[List[V3GlobalEventRulesRulesetTypedDict]]


class V3GlobalEventRulesCreatedGlobalEventRuleResponse(BaseModel):
    id: int

    owner_id: str

    name: str

    description: str

    routing_key: str

    created_at: datetime

    created_by: str

    updated_at: datetime

    updated_by: str

    entity_owner: Optional[V3GlobalEventRulesEntityOwner] = None

    rulesets: Optional[List[V3GlobalEventRulesRuleset]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ServicesAPTAConfigRequestTypedDict(TypedDict):
    is_enabled: bool
    timeout_in_mins: int


class V3ServicesAPTAConfigRequest(BaseModel):
    is_enabled: bool

    timeout_in_mins: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_scheduleresponse import V4ScheduleResponse, V4ScheduleResponseTypedDict
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class SchedulesCreateScheduleResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V4ScheduleResponseTypedDict


class SchedulesCreateScheduleResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V4ScheduleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DetailTypedDict(TypedDict):
    field: str
    message: str


class Detail(BaseModel):
    field: str

    message: str


class ErrorTypedDict(TypedDict):
    code: str
    message: str
    details: NotRequired[List[DetailTypedDict]]


class Error(BaseModel):
    code: str

    message: str

    details: Optional[List[Detail]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .common_v4_pageinfo import CommonV4PageInfo, CommonV4PageInfoTypedDict
from .v4_overrideresponse import V4OverrideResponse, V4OverrideResponseTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class OverridesListOverridesRequestTypedDict(TypedDict):
    schedule_id: str
    start_time: str
    end_time: str
    participant_id: NotRequired[str]
    page_size: NotRequired[int]
    cursor: NotRequired[str]


class OverridesListOverridesRequest(BaseModel):
    schedule_id: Annotated[
        str,
        pydantic.Field(alias="scheduleID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    start_time: Annotated[
        str,
        pydantic.Field(alias="startTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    end_time: Annotated[
        str,
        pydantic.Field(alias="endTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    participant_id: Annotated[
        Optional[str],
        pydantic.Field(alias="participantID"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    page_size: Annotated[
        Optional[int],
        pydantic.Field(alias="pageSize"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    cursor: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None


class OverridesListOverridesResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V4OverrideResponseTypedDict]
    page_info: CommonV4PageInfoTypedDict


class OverridesListOverridesResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[V4OverrideResponse]

    page_info: Annotated[CommonV4PageInfo, pydantic.Field(alias="pageInfo")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3SLOMarkSLOAffectedRequestTypedDict(TypedDict):
    incident_id: str
    slis: List[str]
    error_budget_spent: float
    owner_type: str
    owner_id: str
    org_id: str


class V3SLOMarkSLOAffectedRequest(BaseModel):
    incident_id: str

    slis: List[str]

    error_budget_spent: float

    owner_type: str

    owner_id: str

    org_id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class AdditionalResponderTypedDict(TypedDict):
    id: str
    type: str


class AdditionalResponder(BaseModel):
    id: str

    type: str


class V3IncidentsAdditionalRespondersAddAdditionalRespondersRequestTypedDict(TypedDict):
    r"""Request body for adding additional responders to an incident."""

    additional_responders: List[AdditionalResponderTypedDict]


class V3IncidentsAdditionalRespondersAddAdditionalRespondersRequest(BaseModel):
    r"""Request body for adding additional responders to an incident."""

    additional_responders: List[AdditionalResponder]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_communicationcards_communicationcardresponse import (
    V3IncidentsCommunicationCardsCommunicationCardResponse,
    V3IncidentsCommunicationCardsCommunicationCardResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class CommunicationCardsArchiveSlackChannelResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3IncidentsCommunicationCardsCommunicationCardResponseTypedDict


class CommunicationCardsArchiveSlackChannelResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3IncidentsCommunicationCardsCommunicationCardResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsIncidentExportAsyncResponseTypedDict(TypedDict):
    message: str
    request_id: str


class V3IncidentsIncidentExportAsyncResponse(BaseModel):
    message: str

    request_id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3ServicesOverlayOptInForKeyBasedDeduplicationRequestTypedDict(TypedDict):
    dedup_key_enabled: NotRequired[bool]


class V3ServicesOverlayOptInForKeyBasedDeduplicationRequest(BaseModel):
    dedup_key_enabled: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3WorkflowsUpdateActionsOrderResponseDataTypedDict(TypedDict):
    action_order: List[int]


class V3WorkflowsUpdateActionsOrderResponseData(BaseModel):
    action_order: List[int]


class V3WorkflowsUpdateActionsOrderResponseTypedDict(TypedDict):
    data: V3WorkflowsUpdateActionsOrderResponseDataTypedDict


class V3WorkflowsUpdateActionsOrderResponse(BaseModel):
    data: V3WorkflowsUpdateActionsOrderResponseData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3SquadsUpdateSquadRequestTypedDict(TypedDict):
    name: NotRequired[str]
    members: NotRequired[List[str]]


class V3SquadsUpdateSquadRequest(BaseModel):
    name: Optional[str] = None

    members: Optional[List[str]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3ServicesServiceMaintainerType = Literal[
    "user",
    "squad",
]


class V3ServicesServiceMaintainerTypedDict(TypedDict):
    id: str
    type: V3ServicesServiceMaintainerType


class V3ServicesServiceMaintainer(BaseModel):
    id: str

    type: V3ServicesServiceMaintainerType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_newstatuspagecomponentlist import (
    V4StatusPagesNewStatusPageComponentList,
    V4StatusPagesNewStatusPageComponentListTypedDict,
)
from .v4_statuspages_newstatuspagethemecolor import (
    V4StatusPagesNewStatusPageThemeColor,
    V4StatusPagesNewStatusPageThemeColorTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


V4StatusPagesCreateStatusPageRequestOwnerType = Literal[
    "team",
    "user",
    "squad",
]


class V4StatusPagesCreateStatusPageRequestTypedDict(TypedDict):
    name: str
    domain_name: str
    logo_url: str
    timezone: str
    team_id: str
    contact_email: str
    owner_type: V4StatusPagesCreateStatusPageRequestOwnerType
    owner_id: str
    description: NotRequired[str]
    is_public: NotRequired[bool]
    custom_domain_name: NotRequired[str]
    theme_color: NotRequired[V4StatusPagesNewStatusPageThemeColorTypedDict]
    components: NotRequired[List[V4StatusPagesNewStatusPageComponentListTypedDict]]
    allow_webhook_subscription: NotRequired[bool]
    allow_components_subscription: NotRequired[bool]
    allow_maintenance_subscription: NotRequired[bool]


class V4StatusPagesCreateStatusPageRequest(BaseModel):
    name: str

    domain_name: Annotated[str, pydantic.Field(alias="domainName")]

    logo_url: Annotated[str, pydantic.Field(alias="logoUrl")]

    timezone: str

    team_id: Annotated[str, pydantic.Field(alias="teamID")]

    contact_email: Annotated[str, pydantic.Field(alias="contactEmail")]

    owner_type: Annotated[
        V4StatusPagesCreateStatusPageRequestOwnerType, pydantic.Field(alias="ownerType")
    ]

    owner_id: Annotated[str, pydantic.Field(alias="ownerID")]

    description: Optional[str] = None

    is_public: Annotated[Optional[bool], pydantic.Field(alias="isPublic")] = None

    custom_domain_name: Annotated[
        Optional[str], pydantic.Field(alias="customDomainName")
    ] = None

    theme_color: Annotated[
        Optional[V4StatusPagesNewStatusPageThemeColor],
        pydantic.Field(alias="themeColor"),
    ] = None

    components: Optional[List[V4StatusPagesNewStatusPageComponentList]] = None

    allow_webhook_subscription: Annotated[
        Optional[bool], pydantic.Field(alias="allowWebhookSubscription")
    ] = None

    allow_components_subscription: Annotated[
        Optional[bool], pydantic.Field(alias="allowComponentsSubscription")
    ] = None

    allow_maintenance_subscription: Annotated[
        Optional[bool], pydantic.Field(alias="allowMaintenanceSubscription")
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_createworkflowrequestupdate import (
    V3WorkflowsCreateWorkflowRequestUpdate,
    V3WorkflowsCreateWorkflowRequestUpdateTypedDict,
)
from .v3_workflows_updateworkflowapiresponse import (
    V3WorkflowsUpdateWorkflowAPIResponse,
    V3WorkflowsUpdateWorkflowAPIResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class WorkflowsUpdateWorkflowRequestTypedDict(TypedDict):
    workflow_id: str
    v3_workflows_create_workflow_request_update: (
        V3WorkflowsCreateWorkflowRequestUpdateTypedDict
    )


class WorkflowsUpdateWorkflowRequest(BaseModel):
    workflow_id: Annotated[
        str,
        pydantic.Field(alias="workflowID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_workflows_create_workflow_request_update: Annotated[
        V3WorkflowsCreateWorkflowRequestUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class WorkflowsUpdateWorkflowResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3WorkflowsUpdateWorkflowAPIResponseTypedDict


class WorkflowsUpdateWorkflowResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3WorkflowsUpdateWorkflowAPIResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsIncidentPriorityUpdateResponseTypedDict(TypedDict):
    incident_id: str
    priority: str


class V3IncidentsIncidentPriorityUpdateResponse(BaseModel):
    incident_id: str

    priority: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V4StatusPagesTotalSubscribersCountTypedDict(TypedDict):
    total_count: int


class V4StatusPagesTotalSubscribersCount(BaseModel):
    total_count: Annotated[int, pydantic.Field(alias="totalCount")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_rotationresponse import V4RotationResponse, V4RotationResponseTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing import List
from typing_extensions import Annotated, TypedDict


class RotationsGetScheduleRotationsRequestTypedDict(TypedDict):
    schedule_id: str


class RotationsGetScheduleRotationsRequest(BaseModel):
    schedule_id: Annotated[
        str,
        pydantic.Field(alias="scheduleID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class RotationsGetScheduleRotationsResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V4RotationResponseTypedDict]


class RotationsGetScheduleRotationsResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[V4RotationResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsMsTeamsMessageChannelUpdateName = Literal["msteams_message_channel",]


class V3WorkflowsMsTeamsMessageChannelUpdateDataTypedDict(TypedDict):
    channel_id: NotRequired[str]
    message: NotRequired[str]


class V3WorkflowsMsTeamsMessageChannelUpdateData(BaseModel):
    channel_id: Optional[str] = None

    message: Optional[str] = None


class V3WorkflowsMsTeamsMessageChannelUpdateTypedDict(TypedDict):
    name: V3WorkflowsMsTeamsMessageChannelUpdateName
    data: NotRequired[V3WorkflowsMsTeamsMessageChannelUpdateDataTypedDict]


class V3WorkflowsMsTeamsMessageChannelUpdate(BaseModel):
    name: V3WorkflowsMsTeamsMessageChannelUpdateName

    data: Optional[V3WorkflowsMsTeamsMessageChannelUpdateData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_communicationcards_communicationcardresponse import (
    V3IncidentsCommunicationCardsCommunicationCardResponse,
    V3IncidentsCommunicationCardsCommunicationCardResponseTypedDict,
)
from .v3_incidents_communicationcards_updatecommunicationcardrequest import (
    V3IncidentsCommunicationCardsUpdateCommunicationCardRequest,
    V3IncidentsCommunicationCardsUpdateCommunicationCardRequestTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class CommunicationCardsUpdateCommunicationCardRequestTypedDict(TypedDict):
    incident_id: str
    r"""Required"""
    communication_card_id: str
    v3_incidents_communication_cards_update_communication_card_request: (
        V3IncidentsCommunicationCardsUpdateCommunicationCardRequestTypedDict
    )


class CommunicationCardsUpdateCommunicationCardRequest(BaseModel):
    incident_id: Annotated[
        str,
        pydantic.Field(alias="IncidentId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Required"""

    communication_card_id: Annotated[
        str,
        pydantic.Field(alias="communicationCardId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_incidents_communication_cards_update_communication_card_request: Annotated[
        V3IncidentsCommunicationCardsUpdateCommunicationCardRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class CommunicationCardsUpdateCommunicationCardResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3IncidentsCommunicationCardsCommunicationCardResponseTypedDict


class CommunicationCardsUpdateCommunicationCardResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3IncidentsCommunicationCardsCommunicationCardResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_actionrequest import (
    V3WorkflowsActionRequest,
    V3WorkflowsActionRequestTypedDict,
)
from .v3_workflows_actionresponse import (
    V3WorkflowsActionResponse,
    V3WorkflowsActionResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class WorkflowsCreateActionRequestTypedDict(TypedDict):
    workflow_id: str
    v3_workflows_action_request: V3WorkflowsActionRequestTypedDict


class WorkflowsCreateActionRequest(BaseModel):
    workflow_id: Annotated[
        str,
        pydantic.Field(alias="workflowID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_workflows_action_request: Annotated[
        V3WorkflowsActionRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class WorkflowsCreateActionResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3WorkflowsActionResponseTypedDict


class WorkflowsCreateActionResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3WorkflowsActionResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_serviceresponse import (
    V3ServicesServiceResponse,
    V3ServicesServiceResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class ServicesGetServicesByNameRequestTypedDict(TypedDict):
    name: str
    owner_id: str


class ServicesGetServicesByNameRequest(BaseModel):
    name: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=False))
    ]

    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=False))
    ]


class ServicesGetServicesByNameResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3ServicesServiceResponseTypedDict


class ServicesGetServicesByNameResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3ServicesServiceResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_actionrequest import (
    V3WorkflowsActionRequest,
    V3WorkflowsActionRequestTypedDict,
)
from .v3_workflows_createworkflowfilter import (
    V3WorkflowsCreateWorkflowFilter,
    V3WorkflowsCreateWorkflowFilterTypedDict,
)
from .v3_workflows_entityownerupdate import (
    V3WorkflowsEntityOwnerUpdate,
    V3WorkflowsEntityOwnerUpdateTypedDict,
)
from .v3_workflows_tag import V3WorkflowsTag, V3WorkflowsTagTypedDict
from .v3_workflows_workflowtrigger import V3WorkflowsWorkflowTrigger
from squadcast_sdk.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsCreateWorkflowRequestUpdateOwnerType = Literal[
    "user",
    "squad",
]


class V3WorkflowsCreateWorkflowRequestUpdateTypedDict(TypedDict):
    title: NotRequired[str]
    description: NotRequired[str]
    owner_id: NotRequired[str]
    owner_type: NotRequired[V3WorkflowsCreateWorkflowRequestUpdateOwnerType]
    entity_owner: NotRequired[V3WorkflowsEntityOwnerUpdateTypedDict]
    tags: NotRequired[List[V3WorkflowsTagTypedDict]]
    trigger: NotRequired[V3WorkflowsWorkflowTrigger]
    filters: NotRequired[V3WorkflowsCreateWorkflowFilterTypedDict]
    actions: NotRequired[List[V3WorkflowsActionRequestTypedDict]]
    enabled: NotRequired[bool]


class V3WorkflowsCreateWorkflowRequestUpdate(BaseModel):
    title: Optional[str] = None

    description: Optional[str] = None

    owner_id: Optional[str] = None

    owner_type: Optional[V3WorkflowsCreateWorkflowRequestUpdateOwnerType] = None

    entity_owner: Optional[V3WorkflowsEntityOwnerUpdate] = None

    tags: Optional[List[V3WorkflowsTag]] = None

    trigger: Optional[V3WorkflowsWorkflowTrigger] = None

    filters: Optional[V3WorkflowsCreateWorkflowFilter] = None

    actions: Optional[List[V3WorkflowsActionRequest]] = None

    enabled: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_webforms_createorupdatewebformrequest import (
    V3WebformsCreateOrUpdateWebformRequest,
    V3WebformsCreateOrUpdateWebformRequestTypedDict,
)
from .v3_webforms_webformresponse import (
    V3WebformsWebformResponse,
    V3WebformsWebformResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class WebformsUpdateWebformRequestTypedDict(TypedDict):
    webform_id: int
    v3_webforms_create_or_update_webform_request: (
        V3WebformsCreateOrUpdateWebformRequestTypedDict
    )


class WebformsUpdateWebformRequest(BaseModel):
    webform_id: Annotated[
        int,
        pydantic.Field(alias="webformId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_webforms_create_or_update_webform_request: Annotated[
        V3WebformsCreateOrUpdateWebformRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class WebformsUpdateWebformResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3WebformsWebformResponseTypedDict


class WebformsUpdateWebformResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3WebformsWebformResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class ResolutionReasonTypedDict(TypedDict):
    message: str


class ResolutionReason(BaseModel):
    message: str


class V3IncidentsResolveIncidentRequestTypedDict(TypedDict):
    r"""Request body for resolving an incident."""

    resolution_reason: ResolutionReasonTypedDict


class V3IncidentsResolveIncidentRequest(BaseModel):
    r"""Request body for resolving an incident."""

    resolution_reason: ResolutionReason



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class RequestsStatusTypedDict(TypedDict):
    pass


class RequestsStatus(BaseModel):
    pass


class IncidentsGetIncidentsStatusByRequestidsMetaTypedDict(TypedDict):
    error_count: int
    success_count: int


class IncidentsGetIncidentsStatusByRequestidsMeta(BaseModel):
    error_count: int

    success_count: int


class IncidentsGetIncidentsStatusByRequestidsResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    requests_status: RequestsStatusTypedDict
    meta: IncidentsGetIncidentsStatusByRequestidsMetaTypedDict


class IncidentsGetIncidentsStatusByRequestidsResponse(BaseModel):
    r"""The request has succeeded."""

    requests_status: RequestsStatus

    meta: IncidentsGetIncidentsStatusByRequestidsMeta



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3GlobalEventRulesRuleActionUpdateTypedDict(TypedDict):
    route_to: NotRequired[str]
    r"""The service ID to which the alert should be routed."""


class V3GlobalEventRulesRuleActionUpdate(BaseModel):
    route_to: Optional[str] = None
    r"""The service ID to which the alert should be routed."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_users_apitokenresponse import (
    V3UsersAPITokenResponse,
    V3UsersAPITokenResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class UsersGetAllTokensResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3UsersAPITokenResponseTypedDict]


class UsersGetAllTokensResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[V3UsersAPITokenResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_analytics_analyticsresponse import (
    V3AnalyticsAnalyticsResponse,
    V3AnalyticsAnalyticsResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AnalyticsGetOrgAnalyticsRequestTypedDict(TypedDict):
    from_: str
    to: str
    team_id: NotRequired[str]
    user_id: NotRequired[str]


class AnalyticsGetOrgAnalyticsRequest(BaseModel):
    from_: Annotated[
        str,
        pydantic.Field(alias="from"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    to: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=False))
    ]

    team_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    user_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None


class AnalyticsGetOrgAnalyticsResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3AnalyticsAnalyticsResponseTypedDict


class AnalyticsGetOrgAnalyticsResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3AnalyticsAnalyticsResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_slo_sloactiontype import V3SLOSLOActionType
from datetime import datetime
from pydantic import model_serializer
from squadcast_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3SLOSLOActionTypedDict(TypedDict):
    type: V3SLOSLOActionType
    id: NotRequired[int]
    slo_id: NotRequired[int]
    user_id: NotRequired[str]
    squad_id: NotRequired[str]
    service_id: NotRequired[str]
    owner_type: NotRequired[str]
    owner_id: NotRequired[str]
    created_at: NotRequired[datetime]
    updated_at: NotRequired[datetime]
    deleted_at: NotRequired[Nullable[datetime]]


class V3SLOSLOAction(BaseModel):
    type: V3SLOSLOActionType

    id: Optional[int] = None

    slo_id: Optional[int] = None

    user_id: Optional[str] = None

    squad_id: Optional[str] = None

    service_id: Optional[str] = None

    owner_type: Optional[str] = None

    owner_id: Optional[str] = None

    created_at: Optional[datetime] = None

    updated_at: Optional[datetime] = None

    deleted_at: OptionalNullable[datetime] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "slo_id",
            "user_id",
            "squad_id",
            "service_id",
            "owner_type",
            "owner_id",
            "created_at",
            "updated_at",
            "deleted_at",
        ]
        nullable_fields = ["deleted_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3WorkflowsMsTeamsMessageUserName = Literal["msteams_message_user",]


class V3WorkflowsMsTeamsMessageUserDataTypedDict(TypedDict):
    member_id: str
    message: str


class V3WorkflowsMsTeamsMessageUserData(BaseModel):
    member_id: str

    message: str


class V3WorkflowsMsTeamsMessageUserTypedDict(TypedDict):
    name: V3WorkflowsMsTeamsMessageUserName
    data: V3WorkflowsMsTeamsMessageUserDataTypedDict


class V3WorkflowsMsTeamsMessageUser(BaseModel):
    name: V3WorkflowsMsTeamsMessageUserName

    data: V3WorkflowsMsTeamsMessageUserData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_overlay_overlayresponse import (
    V3ServicesOverlayOverlayResponse,
    V3ServicesOverlayOverlayResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing import List
from typing_extensions import Annotated, TypedDict


class OverlayGetAllDedupKeyOverlayByServiceRequestTypedDict(TypedDict):
    service_id: str


class OverlayGetAllDedupKeyOverlayByServiceRequest(BaseModel):
    service_id: Annotated[
        str,
        pydantic.Field(alias="serviceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class OverlayGetAllDedupKeyOverlayByServiceResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3ServicesOverlayOverlayResponseTypedDict]


class OverlayGetAllDedupKeyOverlayByServiceResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[V3ServicesOverlayOverlayResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ExtensionsWebhooksWebhookListItemTypedDict(TypedDict):
    id: str
    name: str
    description: str
    trigger_type: str
    payload_type: str
    custom_payload_template_name: str


class V3ExtensionsWebhooksWebhookListItem(BaseModel):
    id: str

    name: str

    description: str

    trigger_type: str

    payload_type: str

    custom_payload_template_name: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_slo_slo import V3SloSlo, V3SloSloTypedDict
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import List
from typing_extensions import Annotated, TypedDict


class SLOGetAllSLOsRequestTypedDict(TypedDict):
    owner_id: str
    offset: str
    limit: str


class SLOGetAllSLOsRequest(BaseModel):
    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]

    offset: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]

    limit: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]


class SLOGetAllSLOsMetaTypedDict(TypedDict):
    total: int
    offset: int
    limit: int


class SLOGetAllSLOsMeta(BaseModel):
    total: int

    offset: int

    limit: int


class SLOGetAllSLOsDataTypedDict(TypedDict):
    meta: SLOGetAllSLOsMetaTypedDict
    slos: List[V3SloSloTypedDict]


class SLOGetAllSLOsData(BaseModel):
    meta: SLOGetAllSLOsMeta

    slos: List[V3SloSlo]


class SLOGetAllSLOsResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: SLOGetAllSLOsDataTypedDict


class SLOGetAllSLOsResponse(BaseModel):
    r"""The request has succeeded."""

    data: SLOGetAllSLOsData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_newstatuspagecomponentlist import (
    V4StatusPagesNewStatusPageComponentList,
    V4StatusPagesNewStatusPageComponentListTypedDict,
)
from .v4_statuspages_newstatuspagethemecolor import (
    V4StatusPagesNewStatusPageThemeColor,
    V4StatusPagesNewStatusPageThemeColorTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesCreateStatusPageResponseTypedDict(TypedDict):
    id: int
    organization_id: str
    name: str
    is_public: bool
    timezone: str
    domain_name: str
    contact_email: str
    allow_components_subscription: bool
    allow_maintenance_subscription: bool
    allow_webhook_subscription: bool
    owner_type: str
    owner_id: str
    team_id: str
    description: NotRequired[str]
    theme_color: NotRequired[V4StatusPagesNewStatusPageThemeColorTypedDict]
    components: NotRequired[List[V4StatusPagesNewStatusPageComponentListTypedDict]]


class V4StatusPagesCreateStatusPageResponse(BaseModel):
    id: int

    organization_id: Annotated[str, pydantic.Field(alias="organizationID")]

    name: str

    is_public: Annotated[bool, pydantic.Field(alias="isPublic")]

    timezone: str

    domain_name: Annotated[str, pydantic.Field(alias="domainName")]

    contact_email: Annotated[str, pydantic.Field(alias="contactEmail")]

    allow_components_subscription: Annotated[
        bool, pydantic.Field(alias="allowComponentsSubscription")
    ]

    allow_maintenance_subscription: Annotated[
        bool, pydantic.Field(alias="allowMaintenanceSubscription")
    ]

    allow_webhook_subscription: Annotated[
        bool, pydantic.Field(alias="allowWebhookSubscription")
    ]

    owner_type: Annotated[str, pydantic.Field(alias="ownerType")]

    owner_id: Annotated[str, pydantic.Field(alias="ownerID")]

    team_id: Annotated[str, pydantic.Field(alias="teamID")]

    description: Optional[str] = None

    theme_color: Annotated[
        Optional[V4StatusPagesNewStatusPageThemeColor],
        pydantic.Field(alias="themeColor"),
    ] = None

    components: Optional[List[V4StatusPagesNewStatusPageComponentList]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_participantgroup import V4ParticipantGroup, V4ParticipantGroupTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import Annotated, TypedDict


class V4UpdateRotationParticipantsRequestTypedDict(TypedDict):
    participant_groups: List[V4ParticipantGroupTypedDict]


class V4UpdateRotationParticipantsRequest(BaseModel):
    participant_groups: Annotated[
        List[V4ParticipantGroup], pydantic.Field(alias="participantGroups")
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_squads_createsquadresponse import (
    V4SquadsCreateSquadResponse,
    V4SquadsCreateSquadResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class SquadsCreateSquadV4ResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V4SquadsCreateSquadResponseTypedDict


class SquadsCreateSquadV4Response(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V4SquadsCreateSquadResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_ruleaction import (
    V3GlobalEventRulesRuleAction,
    V3GlobalEventRulesRuleActionTypedDict,
)
from datetime import datetime
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3GlobalEventRulesRulesetTypedDict(TypedDict):
    id: NotRequired[int]
    global_event_rule_id: NotRequired[int]
    alert_source_shortname: NotRequired[str]
    alert_source_version: NotRequired[str]
    ordering: NotRequired[List[int]]
    catch_all_action: NotRequired[V3GlobalEventRulesRuleActionTypedDict]
    created_at: NotRequired[datetime]
    created_by: NotRequired[str]
    updated_at: NotRequired[datetime]
    updated_by: NotRequired[str]


class V3GlobalEventRulesRuleset(BaseModel):
    id: Optional[int] = None

    global_event_rule_id: Optional[int] = None

    alert_source_shortname: Optional[str] = None

    alert_source_version: Optional[str] = None

    ordering: Optional[List[int]] = None

    catch_all_action: Optional[V3GlobalEventRulesRuleAction] = None

    created_at: Optional[datetime] = None

    created_by: Optional[str] = None

    updated_at: Optional[datetime] = None

    updated_by: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from datetime import datetime
from squadcast_sdk import errors, models, utils
from squadcast_sdk._hooks import HookContext
from squadcast_sdk.falsepositive import FalsePositive
from squadcast_sdk.types import OptionalNullable, UNSET
from squadcast_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class SlosSDK(BaseSDK):
    false_positive: FalsePositive

    def __init__(
        self, sdk_config: SDKConfiguration, parent_ref: Optional[object] = None
    ) -> None:
        BaseSDK.__init__(self, sdk_config, parent_ref=parent_ref)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.false_positive = FalsePositive(
            self.sdk_configuration, parent_ref=self.parent_ref
        )

    def list_all(
        self,
        *,
        owner_id: str,
        offset: str,
        limit: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SLOGetAllSLOsData:
        r"""Get All SLOs

        Returns all the SLOs of the passed owner_id in the params.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param owner_id:
        :param offset:
        :param limit:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLOGetAllSLOsRequest(
            owner_id=owner_id,
            offset=offset,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/v3/slo",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_getAllSLOs",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SLOGetAllSLOsResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def list_all_async(
        self,
        *,
        owner_id: str,
        offset: str,
        limit: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SLOGetAllSLOsData:
        r"""Get All SLOs

        Returns all the SLOs of the passed owner_id in the params.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param owner_id:
        :param offset:
        :param limit:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLOGetAllSLOsRequest(
            owner_id=owner_id,
            offset=offset,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/slo",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_getAllSLOs",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SLOGetAllSLOsResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def create(
        self,
        *,
        name: str,
        time_interval_type: models.V3SLOTimeIntervalType,
        service_ids: List[str],
        slis: List[str],
        target_slo: float,
        start_time: datetime,
        end_time: datetime,
        duration_in_days: int,
        owner_type: str,
        owner_id: str,
        slo_owner_id: str,
        slo_owner_type: models.V3SLOSLOOwnerType,
        description: Optional[str] = None,
        tags: Optional[
            Union[
                models.V3SLOCreateSLORequestTags,
                models.V3SLOCreateSLORequestTagsTypedDict,
            ]
        ] = None,
        slo_monitoring_checks: Optional[
            Union[
                List[models.V3SLOSLOMonitoringCheck],
                List[models.V3SLOSLOMonitoringCheckTypedDict],
            ]
        ] = None,
        slo_actions: Optional[
            Union[List[models.V3SLOSLOAction], List[models.V3SLOSLOActionTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SLOCreateSLOData:
        r"""Create SLO

        - This API will create SLO.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param name:
        :param time_interval_type:
        :param service_ids:
        :param slis:
        :param target_slo:
        :param start_time:
        :param end_time:
        :param duration_in_days:
        :param owner_type:
        :param owner_id:
        :param slo_owner_id:
        :param slo_owner_type:
        :param description:
        :param tags:
        :param slo_monitoring_checks:
        :param slo_actions:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3SLOCreateSLORequest(
            name=name,
            description=description,
            time_interval_type=time_interval_type,
            service_ids=service_ids,
            slis=slis,
            target_slo=target_slo,
            start_time=start_time,
            end_time=end_time,
            duration_in_days=duration_in_days,
            tags=utils.get_pydantic_model(
                tags, Optional[models.V3SLOCreateSLORequestTags]
            ),
            slo_monitoring_checks=utils.get_pydantic_model(
                slo_monitoring_checks, Optional[List[models.V3SLOSLOMonitoringCheck]]
            ),
            slo_actions=utils.get_pydantic_model(
                slo_actions, Optional[List[models.V3SLOSLOAction]]
            ),
            owner_type=owner_type,
            owner_id=owner_id,
            slo_owner_id=slo_owner_id,
            slo_owner_type=slo_owner_type,
        )

        req = self._build_request(
            method="POST",
            path="/v3/slo",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V3SLOCreateSLORequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_createSLO",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.SLOCreateSLOResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        name: str,
        time_interval_type: models.V3SLOTimeIntervalType,
        service_ids: List[str],
        slis: List[str],
        target_slo: float,
        start_time: datetime,
        end_time: datetime,
        duration_in_days: int,
        owner_type: str,
        owner_id: str,
        slo_owner_id: str,
        slo_owner_type: models.V3SLOSLOOwnerType,
        description: Optional[str] = None,
        tags: Optional[
            Union[
                models.V3SLOCreateSLORequestTags,
                models.V3SLOCreateSLORequestTagsTypedDict,
            ]
        ] = None,
        slo_monitoring_checks: Optional[
            Union[
                List[models.V3SLOSLOMonitoringCheck],
                List[models.V3SLOSLOMonitoringCheckTypedDict],
            ]
        ] = None,
        slo_actions: Optional[
            Union[List[models.V3SLOSLOAction], List[models.V3SLOSLOActionTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SLOCreateSLOData:
        r"""Create SLO

        - This API will create SLO.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param name:
        :param time_interval_type:
        :param service_ids:
        :param slis:
        :param target_slo:
        :param start_time:
        :param end_time:
        :param duration_in_days:
        :param owner_type:
        :param owner_id:
        :param slo_owner_id:
        :param slo_owner_type:
        :param description:
        :param tags:
        :param slo_monitoring_checks:
        :param slo_actions:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3SLOCreateSLORequest(
            name=name,
            description=description,
            time_interval_type=time_interval_type,
            service_ids=service_ids,
            slis=slis,
            target_slo=target_slo,
            start_time=start_time,
            end_time=end_time,
            duration_in_days=duration_in_days,
            tags=utils.get_pydantic_model(
                tags, Optional[models.V3SLOCreateSLORequestTags]
            ),
            slo_monitoring_checks=utils.get_pydantic_model(
                slo_monitoring_checks, Optional[List[models.V3SLOSLOMonitoringCheck]]
            ),
            slo_actions=utils.get_pydantic_model(
                slo_actions, Optional[List[models.V3SLOSLOAction]]
            ),
            owner_type=owner_type,
            owner_id=owner_id,
            slo_owner_id=slo_owner_id,
            slo_owner_type=slo_owner_type,
        )

        req = self._build_request_async(
            method="POST",
            path="/v3/slo",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V3SLOCreateSLORequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_createSLO",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.SLOCreateSLOResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def update(
        self,
        *,
        slo_id: int,
        owner_id_param: str,
        name: str,
        time_interval_type: models.V3SLOTimeIntervalType,
        service_ids: List[str],
        slis: List[str],
        target_slo: float,
        start_time: datetime,
        end_time: datetime,
        duration_in_days: int,
        owner_type: str,
        owner_id: str,
        slo_owner_id: str,
        slo_owner_type: models.V3SLOSLOOwnerType,
        description: Optional[str] = None,
        tags: Optional[
            Union[
                models.V3SLOCreateSLORequestTags,
                models.V3SLOCreateSLORequestTagsTypedDict,
            ]
        ] = None,
        slo_monitoring_checks: Optional[
            Union[
                List[models.V3SLOSLOMonitoringCheck],
                List[models.V3SLOSLOMonitoringCheckTypedDict],
            ]
        ] = None,
        slo_actions: Optional[
            Union[List[models.V3SLOSLOAction], List[models.V3SLOSLOActionTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SLOUpdateSLOData:
        r"""Update SLO

        - This API will update SLO.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param slo_id:
        :param owner_id_param:
        :param name:
        :param time_interval_type:
        :param service_ids:
        :param slis:
        :param target_slo:
        :param start_time:
        :param end_time:
        :param duration_in_days:
        :param owner_type:
        :param owner_id:
        :param slo_owner_id:
        :param slo_owner_type:
        :param description:
        :param tags:
        :param slo_monitoring_checks:
        :param slo_actions:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLOUpdateSLORequest(
            slo_id=slo_id,
            owner_id_param=owner_id_param,
            v3_slo_create_slo_request=models.V3SLOCreateSLORequest(
                name=name,
                description=description,
                time_interval_type=time_interval_type,
                service_ids=service_ids,
                slis=slis,
                target_slo=target_slo,
                start_time=start_time,
                end_time=end_time,
                duration_in_days=duration_in_days,
                tags=utils.get_pydantic_model(
                    tags, Optional[models.V3SLOCreateSLORequestTags]
                ),
                slo_monitoring_checks=utils.get_pydantic_model(
                    slo_monitoring_checks,
                    Optional[List[models.V3SLOSLOMonitoringCheck]],
                ),
                slo_actions=utils.get_pydantic_model(
                    slo_actions, Optional[List[models.V3SLOSLOAction]]
                ),
                owner_type=owner_type,
                owner_id=owner_id,
                slo_owner_id=slo_owner_id,
                slo_owner_type=slo_owner_type,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v3/slo/{sloID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v3_slo_create_slo_request,
                False,
                False,
                "json",
                models.V3SLOCreateSLORequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_updateSLO",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SLOUpdateSLOResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        slo_id: int,
        owner_id_param: str,
        name: str,
        time_interval_type: models.V3SLOTimeIntervalType,
        service_ids: List[str],
        slis: List[str],
        target_slo: float,
        start_time: datetime,
        end_time: datetime,
        duration_in_days: int,
        owner_type: str,
        owner_id: str,
        slo_owner_id: str,
        slo_owner_type: models.V3SLOSLOOwnerType,
        description: Optional[str] = None,
        tags: Optional[
            Union[
                models.V3SLOCreateSLORequestTags,
                models.V3SLOCreateSLORequestTagsTypedDict,
            ]
        ] = None,
        slo_monitoring_checks: Optional[
            Union[
                List[models.V3SLOSLOMonitoringCheck],
                List[models.V3SLOSLOMonitoringCheckTypedDict],
            ]
        ] = None,
        slo_actions: Optional[
            Union[List[models.V3SLOSLOAction], List[models.V3SLOSLOActionTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SLOUpdateSLOData:
        r"""Update SLO

        - This API will update SLO.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param slo_id:
        :param owner_id_param:
        :param name:
        :param time_interval_type:
        :param service_ids:
        :param slis:
        :param target_slo:
        :param start_time:
        :param end_time:
        :param duration_in_days:
        :param owner_type:
        :param owner_id:
        :param slo_owner_id:
        :param slo_owner_type:
        :param description:
        :param tags:
        :param slo_monitoring_checks:
        :param slo_actions:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLOUpdateSLORequest(
            slo_id=slo_id,
            owner_id_param=owner_id_param,
            v3_slo_create_slo_request=models.V3SLOCreateSLORequest(
                name=name,
                description=description,
                time_interval_type=time_interval_type,
                service_ids=service_ids,
                slis=slis,
                target_slo=target_slo,
                start_time=start_time,
                end_time=end_time,
                duration_in_days=duration_in_days,
                tags=utils.get_pydantic_model(
                    tags, Optional[models.V3SLOCreateSLORequestTags]
                ),
                slo_monitoring_checks=utils.get_pydantic_model(
                    slo_monitoring_checks,
                    Optional[List[models.V3SLOSLOMonitoringCheck]],
                ),
                slo_actions=utils.get_pydantic_model(
                    slo_actions, Optional[List[models.V3SLOSLOAction]]
                ),
                owner_type=owner_type,
                owner_id=owner_id,
                slo_owner_id=slo_owner_id,
                slo_owner_type=slo_owner_type,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v3/slo/{sloID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v3_slo_create_slo_request,
                False,
                False,
                "json",
                models.V3SLOCreateSLORequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_updateSLO",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SLOUpdateSLOResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def remove(
        self,
        *,
        slo_id: int,
        owner_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3SLOSLOResponse:
        r"""Remove SLO

        Remove SLO from passed owner_id (team_id) in the params . Upon sccess the slo will be removed.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param slo_id:
        :param owner_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLORemoveSLORequest(
            slo_id=slo_id,
            owner_id=owner_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v3/slo/{sloID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_removeSLO",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SLORemoveSLOResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def remove_async(
        self,
        *,
        slo_id: int,
        owner_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3SLOSLOResponse:
        r"""Remove SLO

        Remove SLO from passed owner_id (team_id) in the params . Upon sccess the slo will be removed.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param slo_id:
        :param owner_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLORemoveSLORequest(
            slo_id=slo_id,
            owner_id=owner_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v3/slo/{sloID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_removeSLO",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SLORemoveSLOResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get(
        self,
        *,
        slo_id: int,
        owner_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3SLOSLOWithInsightsResponse:
        r"""Get SLO By ID

        Returns a SLO details of the given `sloID` in the request param.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param slo_id:
        :param owner_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLOGetSLOByIDRequest(
            slo_id=slo_id,
            owner_id=owner_id,
        )

        req = self._build_request(
            method="GET",
            path="/v3/slo/{sloID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_getSLOById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SLOGetSLOByIDResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        slo_id: int,
        owner_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3SLOSLOWithInsightsResponse:
        r"""Get SLO By ID

        Returns a SLO details of the given `sloID` in the request param.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param slo_id:
        :param owner_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLOGetSLOByIDRequest(
            slo_id=slo_id,
            owner_id=owner_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/slo/{sloID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_getSLOById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SLOGetSLOByIDResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def mark_affected(
        self,
        *,
        slo_id: int,
        owner_id_param: str,
        incident_id: str,
        slis: List[str],
        error_budget_spent: float,
        owner_type: str,
        owner_id: str,
        org_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SLOMarkSLOAffectedData:
        r"""Mark SLO Affected

        This endpoint is used for mark slo affected.

        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param slo_id:
        :param owner_id_param:
        :param incident_id:
        :param slis:
        :param error_budget_spent:
        :param owner_type:
        :param owner_id:
        :param org_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLOMarkSLOAffectedRequest(
            slo_id=slo_id,
            owner_id_param=owner_id_param,
            v3_slo_mark_slo_affected_request=models.V3SLOMarkSLOAffectedRequest(
                incident_id=incident_id,
                slis=slis,
                error_budget_spent=error_budget_spent,
                owner_type=owner_type,
                owner_id=owner_id,
                org_id=org_id,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v3/slo/{sloID}/incident",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v3_slo_mark_slo_affected_request,
                False,
                False,
                "json",
                models.V3SLOMarkSLOAffectedRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_markSLOAffected",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.SLOMarkSLOAffectedResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def mark_affected_async(
        self,
        *,
        slo_id: int,
        owner_id_param: str,
        incident_id: str,
        slis: List[str],
        error_budget_spent: float,
        owner_type: str,
        owner_id: str,
        org_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SLOMarkSLOAffectedData:
        r"""Mark SLO Affected

        This endpoint is used for mark slo affected.

        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param slo_id:
        :param owner_id_param:
        :param incident_id:
        :param slis:
        :param error_budget_spent:
        :param owner_type:
        :param owner_id:
        :param org_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SLOMarkSLOAffectedRequest(
            slo_id=slo_id,
            owner_id_param=owner_id_param,
            v3_slo_mark_slo_affected_request=models.V3SLOMarkSLOAffectedRequest(
                incident_id=incident_id,
                slis=slis,
                error_budget_spent=error_budget_spent,
                owner_type=owner_type,
                owner_id=owner_id,
                org_id=org_id,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v3/slo/{sloID}/incident",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v3_slo_mark_slo_affected_request,
                False,
                False,
                "json",
                models.V3SLOMarkSLOAffectedRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SLO_markSLOAffected",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.SLOMarkSLOAffectedResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal, Union
from typing_extensions import TypeAliasType


V3ExtensionsMSTeamsEventClassEnum = Literal[
    "incident_triggered",
    "incident_retriggered",
    "incident_acknowledged",
    "incident_resolved",
    "incident_reassigned",
    "incident_notes_added",
    "incident_postmortem_started",
    "incident_priority_updated",
    "incident_snoozed",
    "incident_unsnoozed",
    "incident_delayed_notifications_resumed",
]


V3ExtensionsMSTeamsEventClassTypedDict = TypeAliasType(
    "V3ExtensionsMSTeamsEventClassTypedDict",
    Union[str, V3ExtensionsMSTeamsEventClassEnum],
)
r"""Represents the specific type of an incident-related event."""


V3ExtensionsMSTeamsEventClass = TypeAliasType(
    "V3ExtensionsMSTeamsEventClass", Union[str, V3ExtensionsMSTeamsEventClassEnum]
)
r"""Represents the specific type of an incident-related event."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_createdglobaleventruleresponse import (
    V3GlobalEventRulesCreatedGlobalEventRuleResponse,
    V3GlobalEventRulesCreatedGlobalEventRuleResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class GlobalEventRulesCreateGlobalEventRuleResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3GlobalEventRulesCreatedGlobalEventRuleResponseTypedDict


class GlobalEventRulesCreateGlobalEventRuleResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3GlobalEventRulesCreatedGlobalEventRuleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_analyticsmetrics import (
    V3IncidentsAnalyticsMetrics,
    V3IncidentsAnalyticsMetricsTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsAnalyticsTypedDict(TypedDict):
    r"""Incident analytics data."""

    tta: V3IncidentsAnalyticsMetricsTypedDict
    r"""Analytics metrics for TTA (Time to Acknowledge) and TTR (Time to Resolve)."""
    ttr: V3IncidentsAnalyticsMetricsTypedDict
    r"""Analytics metrics for TTA (Time to Acknowledge) and TTR (Time to Resolve)."""


class V3IncidentsAnalytics(BaseModel):
    r"""Incident analytics data."""

    tta: V3IncidentsAnalyticsMetrics
    r"""Analytics metrics for TTA (Time to Acknowledge) and TTR (Time to Resolve)."""

    ttr: V3IncidentsAnalyticsMetrics
    r"""Analytics metrics for TTA (Time to Acknowledge) and TTR (Time to Resolve)."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_teams_abilities import V3TeamsAbilities, V3TeamsAbilitiesTypedDict
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3TeamsTeamRoleResponseTypedDict(TypedDict):
    id: str
    name: str
    slug: str
    default: bool
    abilities: NotRequired[V3TeamsAbilitiesTypedDict]


class V3TeamsTeamRoleResponse(BaseModel):
    id: str

    name: str

    slug: str

    default: bool

    abilities: Optional[V3TeamsAbilities] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3IncidentsAdditionalRespondersAdditionalResponderTypedDict(TypedDict):
    r"""Represents an additional responder for an incident."""

    id: str
    type: str
    name: NotRequired[str]
    time_of_assignment: NotRequired[str]


class V3IncidentsAdditionalRespondersAdditionalResponder(BaseModel):
    r"""Represents an additional responder for an incident."""

    id: str

    type: str

    name: Optional[str] = None

    time_of_assignment: Optional[str] = None
